# $File: //member/autrijus/only-latest/t/1-basic.t $ $Author: autrijus $
# $Revision: #1 $ $Change: 8676 $ $only: 2003/11/01 06:14:05 $

use strict;
use Test;
BEGIN { plan tests => 1 }

use only::latest; ok(1);

1;
